/**********************************************************

This software is part of J.-S. Caux's ABACUS library.

Copyright (c) J.-S. Caux.

-----------------------------------------------------------

File:  2CBG_ThLim.cc

Purpose:  solves the TBA equations for the 2-component Bose gas

***********************************************************/

#include "ABACUS.h"

using namespace std;
using namespace ABACUS;

int main(int argc, const char* argv[])
{

  if (argc != 7) ABACUSerror("Wrong number of arguments to 2CBG_ThLim executable.  "
			     "Use c (best to set to 1), mu, Omega, kBT, TT(minutes), "
			     "bool Save_data (0 == false).");

  DP c_int = atof(argv[1]);
  DP mu = atof(argv[2]);
  DP Omega = atof(argv[3]);
  DP kBT = atof(argv[4]);
  int Max_Secs = 60 * atoi(argv[5]);
  bool Save_data = bool(atoi(argv[6]));

  if (c_int <= 0.0) ABACUSerror("Give a strictly positive c.");
  if (Omega <= 0.0) ABACUSerror("Give a strictly positive Omega, "
				"otherwise the algorithm cannot converge.");
  if (kBT <= 0.0) ABACUSerror("Negative T ?  You must be a string theorist.");
  if (Max_Secs < 10) ABACUSerror("Give more time.");

  Solve_2CBG_TBAE_via_refinements (c_int, mu, Omega, kBT, Max_Secs, Save_data);

  return(0);
}
