/**********************************************************

This software is part of J.-S. Caux's ABACUS library.

Copyright (c) J.-S. Caux

-----------------------------------------------------------

File:  Heis_DSF.cc

Purpose:  main function for ABACUS for Heisenberg spin-1/2 chain

***********************************************************/

#include "ABACUS.h"

using namespace std;
using namespace ABACUS;


int main(int argc, char* argv[])
{

  if (argc != 8) { // provide some info

    cout << endl << "Welcome to ABACUS\t(copyright J.-S. Caux)." << endl;
    cout << endl << "Usage of Heis_DSF executable: " << endl;
    cout << endl << "Provide the following arguments:" << endl << endl;
    cout << "char whichDSF \t\t Which structure factor should be calculated ?  "
      "Options are:  m for S- S+, z for Sz Sz, p for S+ S-." << endl;
    cout << "DP Delta \t\t Value of the anisotropy:  use positive real values only" << endl;
    cout << "int N \t\t\t Length (number of sites) of the system:  "
      "use positive even integer values only" << endl;
    cout << "int M \t\t\t Number of down spins:  use positive integer values between 1 and N/2" << endl;
    cout << "int Max_Secs \t\t Allowed computational time:  (in seconds)" << endl;
    cout << "DP target_sumrule \t sumrule saturation you're satisfied with" << endl;
    cout << "bool refine \t\t Is this a refinement of a earlier calculations ?  (0 == false, 1 == true)" << endl;
    cout << endl << "EXAMPLE: " << endl << endl;
    cout << "Heis_DSF z 1.0 100 40 0 100 600 1.0 0" << endl << endl;
  }

  else if (argc == 8) { // !fixed_iK
    int ctr = 1;
    char whichDSF = *argv[ctr++];
    DP Delta = atof(argv[ctr++]);
    int N = atoi(argv[ctr++]);
    int M = atoi(argv[ctr++]);
    int Max_Secs = atoi(argv[ctr++]);
    DP target_sumrule = atof(argv[ctr++]);
    bool refine = (atoi(argv[ctr++]) == 1);

    // We systematically scan over all momentum integers (to avoid problems with Brillouin folding
    int iKmin = -1000*N;
    int iKmax = 1000*N;
    Scan_Heis (whichDSF, Delta, N, M, iKmin, iKmax, Max_Secs, target_sumrule, refine);
  }

  return(0);
}
